@@ -5,7 +5,7 @@
  * 	Mike Gemuende <mike@gemuende.de>
  * 	Jim Nelson <jim@yorba.org>
  *
- * This is free software. See COPYING for details.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef GEXIV2_METADATA_H
@@ -19,19 +19,53 @@
 
 /**
  * SECTION: gexiv2-metadata
+ * @title: Generic image metadata handling
  * @short_description: Class to handle the various image metadata standards
+ * @include: gexiv2/gexiv2.h
  *
  * #GExiv2Metadata is a generic object that provides everything from simple
  * aggregated accessors to common data such as image comments up to
- * fine-grained access to specifig tags of a specific format, be it EXIF, IPTC
+ * fine-grained access to specific tags of a specific format, be it EXIF, IPTC
  * or XMP
  */
 
+/**
+ * SECTION: gexiv2-metadata-xmp
+ * @short_description: Functionality specific to the XMP familiy of metadata information
+ * @title: Functions related to XMP metadata
+ * @include: gexiv2/gexiv2.h
+ *
+ * While being part of #GExiv2Metadata, these functions are dealing with specific
+ * aspects of XMP
+ */
+
+/**
+ * SECTION: gexiv2-metadata-exif
+ * @short_description: Functionality specific to the EXIF familiy of metadata information
+ * @title: Functions related to EXIF metadata
+ * @include: gexiv2/gexiv2.h
+ *
+ * While being part of #GExiv2Metadata, these functions are dealing with specific
+ * aspects of EXIF
+ */
+
+/**
+ * SECTION: gexiv2-metadata-iptc
+ * @short_description: Functionality specific to the IPTC familiy of metadata information
+ * @title: Functions related to IPTC metadata
+ * @include: gexiv2/gexiv2.h
+ *
+ * While being part of #GExiv2Metadata, these functions are dealing with specific
+ * aspects of IPTC
+ */
+
 /**
  * SECTION: gexiv2-preview-image
+ * @title: Preview images
  * @short_description: Class describing preview images (or thumbnails) that
  * are part of the metadata.
  * @see_also: #GExiv2Metadata #GExiv2PreviewProperties
+ * @include: gexiv2/gexiv2.h
  *
  * #GExiv2PreviewImage is an accessor to the preview images contained in
  * the image's metadata. This could be anything from a thumbnail to a
@@ -42,7 +76,7 @@
  * to be fetched.
  * <informalexample><programlisting>
  * GExiv2PreviewProperties **properties, **it;
- * properts = it = gexiv2_metadata_get_preview_properties(metadata);
+ * properties = it = gexiv2_metadata_get_preview_properties(metadata);
  *
  * while (*it) {
  *   preview_image = gexiv2_metadata_get_preview_image(metadata, *it);
@@ -53,12 +87,36 @@
 
 /**
  * SECTION: gexiv2-preview-properties
+ * @title: Properties of preview images
  * @short_description: Class describing properties of a preview image
  * @see_also: #GExiv2Metadata #GExiv2PreviewProperties
  *
  * Metadata can contain multiple preview images. #GExiv2PreviewProperties are
  * used to describe the available image without the need of fetching the whole
- * image from the metadta.
+ * image from the metadata.
+ */
+
+/**
+ * SECTION: gexiv2-version
+ * @title: Library version information
+ * @short_description: Utilities for querying the library version
+ *
+ * Provides macros and functions to query the version of the GExiv2 library during compile- and runtime.
+ */
+
+/**
+ * SECTION: gexiv2-io
+ * @title: Reading and writing metadata
+ * @short_description: How to read and write metadata from files and memory
+ */
+
+/**
+ * SECTION: gexiv2-convenience
+ * @title: High-level functions for specific information
+ * @short_description: A set of helper functions to simplify accessing specific information
+ *
+ * The functions in this section will use several individual tags to provide or set
+ * the required information while trying to follow the MWG recommendations
  */
 
 G_BEGIN_DECLS
@@ -102,9 +160,9 @@ G_BEGIN_DECLS
  * <ulink url="http://jpegclub.org/exif_orientation.html"></ulink>
  */
 typedef enum {
-    /*<private>*/
+	/*< private >*/
 	GEXIV2_ORIENTATION_MIN			= 0,
-    /*<public>*/
+	/*< public >*/
 	GEXIV2_ORIENTATION_UNSPECIFIED	= 0,
 	GEXIV2_ORIENTATION_NORMAL		= 1,
 	GEXIV2_ORIENTATION_HFLIP		= 2,
@@ -114,7 +172,7 @@ typedef enum {
 	GEXIV2_ORIENTATION_ROT_90		= 6,
 	GEXIV2_ORIENTATION_ROT_90_VFLIP	= 7,
 	GEXIV2_ORIENTATION_ROT_270		= 8,
-    /*<private>*/
+	/*< private >*/
 	GEXIV2_ORIENTATION_MAX			= 8
 } GExiv2Orientation;
 
@@ -150,22 +208,34 @@ typedef enum {
  * Taken from: exiv2/src/xmp.hpp
  *
  */
-typedef enum {
-  GEXIV2_OMIT_PACKET_WRAPPER   = 0x0010UL,  //!< Omit the XML packet wrapper.
-  GEXIV2_READ_ONLY_PACKET      = 0x0020UL,  //!< Default is a writeable packet.
-  GEXIV2_USE_COMPACT_FORMAT    = 0x0040UL,  //!< Use a compact form of RDF.
-  GEXIV2_INCLUDE_THUMBNAIL_PAD = 0x0100UL,  //!< Include a padding allowance for a thumbnail image.
-  GEXIV2_EXACT_PACKET_LENGTH   = 0x0200UL,  //!< The padding parameter is the overall packet length.
-  GEXIV2_WRITE_ALIAS_COMMENTS  = 0x0400UL,  //!< Show aliases as XML comments.
-  GEXIV2_OMIT_ALL_FORMATTING   = 0x0800UL   //!< Omit all formatting whitespace.
+typedef enum { /*< flags >*/
+  GEXIV2_OMIT_PACKET_WRAPPER   = 0x0010UL,
+  GEXIV2_READ_ONLY_PACKET      = 0x0020UL,
+  GEXIV2_USE_COMPACT_FORMAT    = 0x0040UL,
+  GEXIV2_INCLUDE_THUMBNAIL_PAD = 0x0100UL,
+  GEXIV2_EXACT_PACKET_LENGTH   = 0x0200UL,
+  GEXIV2_WRITE_ALIAS_COMMENTS  = 0x0400UL,
+  GEXIV2_OMIT_ALL_FORMATTING   = 0x0800UL
 } GExiv2XmpFormatFlags;
 
+/**
+ * GExiv2ByteOrder:
+ * @GEXIV2_BYTE_ORDER_LITTLE: Use little-endian byte order
+ * @GEXIV2_BYTE_ORDER_BIG: Use big-endian byte order
+ *
+ * Options to control the byte order of binary EXIF data exports
+ */
+typedef enum {
+  GEXIV2_BYTE_ORDER_LITTLE = 0,
+  GEXIV2_BYTE_ORDER_BIG = 1
+} GExiv2ByteOrder;
+
 /**
  * GExiv2Metadata:
  *
  * An object holding all the Exiv2 metadata.  Previews, if present, are also available.
  *
- * As gexiv2 is only a wrapper around Exiv2, it's better to read its documentaiton to understand
+ * As gexiv2 is only a wrapper around Exiv2, it's better to read its documentation to understand
  * the full scope of what it offers: <ulink url="http://www.exiv2.org/"></ulink>
  *
  * In particular, rather than providing a getter/setter method pair for every metadata value
@@ -181,6 +251,8 @@ typedef struct _GExiv2Metadata			GExiv2Metadata;
 typedef struct _GExiv2MetadataClass		GExiv2MetadataClass;
 typedef struct _GExiv2MetadataPrivate	GExiv2MetadataPrivate;
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GExiv2Metadata, g_object_unref)
+
 struct _GExiv2Metadata
 {
 	GObject parent_instance;
@@ -253,6 +325,19 @@ gboolean		gexiv2_metadata_open_buf			(GExiv2Metadata *self, const guint8 *data,
  */
 gboolean		gexiv2_metadata_open_stream			(GExiv2Metadata *self, ManagedStreamCallbacks* cb, GError **error);
 
+
+/**
+ * gexiv2_metadata_from_stream:
+ * @self: An instance of #GExiv2Metadata
+ * @stream: A #GInputStream to get meta-data from
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Read meta-data from a GIO stream
+ *
+ * Returns: Boolean success indicator
+ */
+gboolean		gexiv2_metadata_from_stream			(GExiv2Metadata *self, GInputStream* stream, GError **error);
+
 /**
  * gexiv2_metadata_from_app1_segment:
  * @self: An instance of #GExiv2Metadata
@@ -369,6 +454,19 @@ gboolean		gexiv2_metadata_is_iptc_tag				(const gchar* tag);
  */
 gboolean		gexiv2_metadata_is_xmp_tag				(const gchar* tag);
 
+/**
+ * gexiv2_metadata_try_get_tag_label:
+ * @tag: An Exiv2 tag
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: (transfer none) (allow-none): The tag's label
+ *
+ * Since: 0.12.2
+ */
+const gchar*	gexiv2_metadata_try_get_tag_label		(const gchar *tag, GError **error);
+
 /**
  * gexiv2_metadata_get_tag_label:
  * @tag: An Exiv2 tag
@@ -376,9 +474,25 @@ gboolean		gexiv2_metadata_is_xmp_tag				(const gchar* tag);
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: (transfer none) (allow-none): The tag's label
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_label() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_label)
 const gchar*	gexiv2_metadata_get_tag_label		(const gchar *tag);
 
+/**
+ * gexiv2_metadata_try_get_tag_description:
+ * @tag: An Exiv2 tag
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: (transfer none) (allow-none): The tag's description
+ *
+ * Since: 0.12.2
+ */
+const gchar*	gexiv2_metadata_try_get_tag_description	(const gchar *tag, GError **error);
+
 /**
  * gexiv2_metadata_get_tag_description:
  * @tag: An Exiv2 tag
@@ -386,9 +500,28 @@ const gchar*	gexiv2_metadata_get_tag_label		(const gchar *tag);
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: (transfer none) (allow-none): The tag's description
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_description() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_description)
 const gchar*	gexiv2_metadata_get_tag_description	(const gchar *tag);
 
+/**
+ * gexiv2_metadata_try_get_tag_type:
+ * @tag: An Exiv2 tag
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The names of the various Exiv2 tag types can be found at Exiv2::TypeId,
+ * <ulink url="http://exiv2.org/doc/namespaceExiv2.html#5153319711f35fe81cbc13f4b852450c"></ulink>
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: (transfer none) (allow-none): The tag's type name.
+ *
+ * Since: 0.12.2
+ */
+const gchar*	gexiv2_metadata_try_get_tag_type	(const gchar *tag, GError **error);
+
 /**
  * gexiv2_metadata_get_tag_type:
  * @tag: An Exiv2 tag
@@ -399,14 +532,17 @@ const gchar*	gexiv2_metadata_get_tag_description	(const gchar *tag);
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: (transfer none) (allow-none): The tag's type name.
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_type() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_type)
 const gchar*	gexiv2_metadata_get_tag_type	(const gchar *tag);
 
 /**
  * gexiv2_metadata_get_supports_exif:
  * @self: An instance of #GExiv2Metadata
  *
- * Query @self whehter it suports writing EXIF metadata.
+ * Query @self whether it supports writing EXIF metadata.
  *
  * Returns: %TRUE if the loaded image type supports writing EXIF metadata.
  */
@@ -465,6 +601,35 @@ gint			gexiv2_metadata_get_pixel_width		(GExiv2Metadata *self);
  */
 gint			gexiv2_metadata_get_pixel_height	(GExiv2Metadata *self);
 
+/**
+ * gexiv2_metadata_try_get_tag_string:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: (transfer full) (allow-none): The tag's value as a string
+ *
+ * Since: 0.12.2
+ */
+gchar*			gexiv2_metadata_try_get_tag_string	(GExiv2Metadata *self, const gchar* tag, GError **error);
+
+/**
+ * gexiv2_metadata_try_set_tag_string:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @value: The value to set or replace the existing value
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: TRUE on success
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_set_tag_string	(GExiv2Metadata *self, const gchar* tag, const gchar* value, GError **error);
+
 /**
  * gexiv2_metadata_get_tag_string:
  * @self: An instance of #GExiv2Metadata
@@ -472,8 +637,15 @@ gint			gexiv2_metadata_get_pixel_height	(GExiv2Metadata *self);
  *
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
+ * In case of error, a GLib warning will be logged. Use instead
+ * gexiv2_metadata_try_get_tag_string() if you want to avoid this and
+ * control if and how the error is outputted.
+ *
  * Returns: (transfer full) (allow-none): The tag's value as a string
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_string() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_string)
 gchar*			gexiv2_metadata_get_tag_string		(GExiv2Metadata *self, const gchar* tag);
 
 /**
@@ -484,12 +656,34 @@ gchar*			gexiv2_metadata_get_tag_string		(GExiv2Metadata *self, const gchar* tag
  *
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
+ * In case of error, a GLib warning will be logged. Use instead
+ * gexiv2_metadata_try_set_tag_string() if you want to avoid this and
+ * control if and how the error is outputted.
+ *
  * Returns: TRUE on success
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_set_tag_string() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_set_tag_string)
 gboolean		gexiv2_metadata_set_tag_string		(GExiv2Metadata *self, const gchar* tag, const gchar* value);
 
 /**
- * gexiv2_metadata_set_tag_struct:
+ * gexiv2_metadata_try_set_xmp_tag_struct:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @type: The GExiv2StructureType specifying the type of structure
+ * @error: (allow-none): A return location for a #GError or %nullptr
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: TRUE on success
+ *
+ * Since: 0.12.2
+ */
+gboolean gexiv2_metadata_try_set_xmp_tag_struct (GExiv2Metadata *self, const gchar* tag, GExiv2StructureType type, GError **error);
+
+/**
+ * gexiv2_metadata_set_xmp_tag_struct:
  * @self: An instance of #GExiv2Metadata
  * @tag: Exiv2 tag name
  * @type: The GExiv2StructureType specifying the type of structure
@@ -497,9 +691,29 @@ gboolean		gexiv2_metadata_set_tag_string		(GExiv2Metadata *self, const gchar* ta
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: TRUE on success
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_set_xmp_tag_struct() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_set_xmp_tag_struct)
 gboolean gexiv2_metadata_set_xmp_tag_struct (GExiv2Metadata *self, const gchar* tag, GExiv2StructureType type);
 
+/**
+ * gexiv2_metadata_try_get_tag_interpreted_string:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @error: (allow-none): A return location for a #GError or %nullptr
+ *
+ * An interpreted string is one fit for user display.  It may display units or use formatting
+ * appropriate to the type of data the tag holds.
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: (transfer full) (allow-none): The tag's interpreted value as a string
+ *
+ * Since: 0.12.2
+ */
+gchar*			gexiv2_metadata_try_get_tag_interpreted_string (GExiv2Metadata *self, const gchar* tag, GError **error);
+
 /**
  * gexiv2_metadata_get_tag_interpreted_string:
  * @self: An instance of #GExiv2Metadata
@@ -511,9 +725,26 @@ gboolean gexiv2_metadata_set_xmp_tag_struct (GExiv2Metadata *self, const gchar*
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: (transfer full) (allow-none): The tag's interpreted value as a string
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_interpreted_string() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_interpreted_string)
 gchar*			gexiv2_metadata_get_tag_interpreted_string (GExiv2Metadata *self, const gchar* tag);
 
+/**
+ * gexiv2_metadata_try_get_tag_long:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @error: (allow-none): A return location for a #GError or %nullptr
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: The tag's value as a glong
+ *
+ * Since: 0.12.2
+ */
+glong			gexiv2_metadata_try_get_tag_long	(GExiv2Metadata *self, const gchar* tag, GError **error);
+
 /**
  * gexiv2_metadata_get_tag_long:
  * @self: An instance of #GExiv2Metadata
@@ -522,9 +753,27 @@ gchar*			gexiv2_metadata_get_tag_interpreted_string (GExiv2Metadata *self, const
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: The tag's value as a glong
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_long() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_long)
 glong			gexiv2_metadata_get_tag_long		(GExiv2Metadata *self, const gchar* tag);
 
+/**
+ * gexiv2_metadata_try_set_tag_long:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @value: The value to set or replace the existing value
+ * @error: (allow-none): A return location for a #GError or %nullptr
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: TRUE on success
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_set_tag_long	(GExiv2Metadata *self, const gchar* tag, glong value, GError **error);
+
 /**
  * gexiv2_metadata_set_tag_long:
  * @self: An instance of #GExiv2Metadata
@@ -534,9 +783,43 @@ glong			gexiv2_metadata_get_tag_long		(GExiv2Metadata *self, const gchar* tag);
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: TRUE on success
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_set_tag_long() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_set_tag_long)
 gboolean		gexiv2_metadata_set_tag_long		(GExiv2Metadata *self, const gchar* tag, glong value);
 
+/**
+ * gexiv2_metadata_try_get_tag_multiple:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: (transfer full) (allow-none) (array zero-terminated=1): The multiple string values of
+ * the tag.  Returns NULL if parameters are NULL or @tag does not begin with recognised type of
+ * metadata ("Exif.", "Xmp." or "Iptc.").  For a well formed @tag, returns array[0] = NULL if @tag
+ * is undefined or is not set in the current metadata.
+ *
+ * Since: 0.12.2
+ */
+gchar**			gexiv2_metadata_try_get_tag_multiple	(GExiv2Metadata *self, const gchar* tag, GError **error);
+
+/**
+ * gexiv2_metadata_try_set_tag_multiple:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @values: (array zero-terminated=1): An array of values to set or replace the existing value(s)
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: Boolean success value
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_set_tag_multiple	(GExiv2Metadata *self, const gchar* tag, const gchar** values, GError **error);
 
 /**
  * gexiv2_metadata_get_tag_multiple:
@@ -545,9 +828,20 @@ gboolean		gexiv2_metadata_set_tag_long		(GExiv2Metadata *self, const gchar* tag,
  *
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
+ * In case of error, a GLib warning will be logged. Use instead
+ * gexiv2_metadata_try_get_tag_multiple() if you want to avoid this and
+ * control if and how the error is outputted.
+ *
  * Returns: (transfer full) (allow-none) (array zero-terminated=1): The multiple string values of
- * the tag
+ * the tag.  Returns NULL if parameters are NULL or @tag does not begin with recognised type of
+ * metadata ("Exif.", "Xmp." or "Iptc.").  For a well formed @tag, returns array[0] = NULL if @tag
+ * is undefined or is not set in the current metadata.
+ * (Note: <ulink url="https://gitlab.gnome.org/GNOME/gexiv2/-/issues/61">xmpText/langAlt bug</ulink>
+ *  is fixed in gexiv2_metadata_try_get_tag_multiple())
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_multiple() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_multiple)
 gchar**			gexiv2_metadata_get_tag_multiple	(GExiv2Metadata *self, const gchar* tag);
 
 /**
@@ -558,10 +852,31 @@ gchar**			gexiv2_metadata_get_tag_multiple	(GExiv2Metadata *self, const gchar* t
  *
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
+ * In case of error, a GLib warning will be logged. Use instead
+ * gexiv2_metadata_try_set_tag_multiple() if you want to avoid this and
+ * control if and how the error is outputted.
+ *
  * Returns: Boolean success value
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_set_tag_multiple() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_set_tag_multiple)
 gboolean		gexiv2_metadata_set_tag_multiple	(GExiv2Metadata *self, const gchar* tag, const gchar** values);
 
+/**
+ * gexiv2_metadata_try_get_tag_raw:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: Exiv2 tag name
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
+ *
+ * Returns: (transfer full) (allow-none): The tag's raw value as a byte array
+ *
+ * Since: 0.12.2
+ */
+GBytes*			gexiv2_metadata_try_get_tag_raw		(GExiv2Metadata *self, const gchar* tag, GError **error);
+
 /**
  * gexiv2_metadata_get_tag_raw:
  * @self: An instance of #GExiv2Metadata
@@ -570,7 +885,10 @@ gboolean		gexiv2_metadata_set_tag_multiple	(GExiv2Metadata *self, const gchar* t
  * The Exiv2 Tag Reference can be found at <ulink url="http://exiv2.org/metadata.html"></ulink>
  *
  * Returns: (transfer full) (allow-none): The tag's raw value as a byte array
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_tag_raw() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_tag_raw)
 GBytes*			gexiv2_metadata_get_tag_raw			(GExiv2Metadata *self, const gchar* tag);
 
 /*
@@ -599,11 +917,45 @@ void			gexiv2_metadata_clear_exif			(GExiv2Metadata *self);
  *
  * Query @self for a list of available EXIF tags
  *
- * Returns: (transfer full) (array zero-terminated=1): A list of the available EXIF tags in the
+ * Returns: (transfer full) (array zero-terminated=1): A unique list of the available EXIF tags in the
  * loaded image
  */
 gchar**			gexiv2_metadata_get_exif_tags		(GExiv2Metadata *self);
 
+/**
+ * gexiv2_metadata_try_get_exif_tag_rational:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: (in): The tag you want the rational value for
+ * @nom: (out): The numerator
+ * @den: (out): The denominator
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Fetch EXIF @tag represented by a fraction. @nom will contain the numerator,
+ * @den the denominator of the fraction on successful return.
+ *
+ * Returns: (skip): Boolean success value
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_get_exif_tag_rational (GExiv2Metadata *self, const gchar* tag, gint* nom, gint* den, GError **error);
+
+/**
+ * gexiv2_metadata_try_set_exif_tag_rational:
+ * @self: An instance of #GExiv2Metadata
+ * @tag: (in): The Exiv2 tag
+ * @nom: Rational numerator
+ * @den: Rational denominator
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Set EXIF @tag represented by a fraction, with @nom being the numerator,
+ * @den the denominator of the fraction.
+ *
+ * Returns: (skip): Boolean success value
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_set_exif_tag_rational (GExiv2Metadata *self, const gchar* tag, gint nom, gint den, GError **error);
+
 /**
  * gexiv2_metadata_get_exif_tag_rational:
  * @self: An instance of #GExiv2Metadata
@@ -611,11 +963,14 @@ gchar**			gexiv2_metadata_get_exif_tags		(GExiv2Metadata *self);
  * @nom: (out): The numerator
  * @den: (out): The denominator
  *
- * Fetch EXIF @tag represented by a fraction. @nom will contian the numerator,
+ * Fetch EXIF @tag represented by a fraction. @nom will contain the numerator,
  * @den the denominator of the fraction on successful return.
  *
  * Returns: (skip): Boolean success value
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_exif_tag_rational() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_exif_tag_rational)
 gboolean		gexiv2_metadata_get_exif_tag_rational (GExiv2Metadata *self, const gchar* tag, gint* nom, gint* den);
 
 /**
@@ -629,7 +984,10 @@ gboolean		gexiv2_metadata_get_exif_tag_rational (GExiv2Metadata *self, const gch
  * @den the denominator of the fraction.
  *
  * Returns: (skip): Boolean success value
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_set_exif_tag_rational() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_set_exif_tag_rational)
 gboolean		gexiv2_metadata_set_exif_tag_rational (GExiv2Metadata *self, const gchar* tag, gint nom, gint den);
 
 /**
@@ -672,6 +1030,17 @@ void			gexiv2_metadata_set_exif_thumbnail_from_buffer (GExiv2Metadata *self, con
  */
 void			gexiv2_metadata_erase_exif_thumbnail (GExiv2Metadata *self);
 
+/**
+ * gexiv2_metadata_get_exif_data:
+ * @self: An instance of #GExiv2Metadata
+ * @byte_order: Whether to export the data in little or big endian format
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Returns: (transfer full) (allow-none): The content of the EXIF data or %NULL on error
+ *
+ * Since: 0.12.2
+ */
+GBytes *        gexiv2_metadata_get_exif_data (GExiv2Metadata *self, GExiv2ByteOrder byte_order, GError **error);
 
 /*
  * XMP functions
@@ -693,6 +1062,21 @@ gboolean		gexiv2_metadata_has_xmp				(GExiv2Metadata *self);
  */
 void			gexiv2_metadata_clear_xmp			(GExiv2Metadata *self);
 
+/**
+ * gexiv2_metadata_try_generate_xmp_packet:
+ * @self: An instance of #GExiv2Metadata
+ * @xmp_format_flags: One of #GExiv2XmpFormatFlags
+ * @padding: The padding (FIXME: Add documentation)
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Encode the XMP packet as a %NULL-terminated string.
+ *
+ * Returns: (transfer full) (allow-none): Encode the XMP packet and return as a %NULL-terminated string.
+ *
+ * Since: 0.12.2
+ */
+gchar*		gexiv2_metadata_try_generate_xmp_packet	(GExiv2Metadata *self, GExiv2XmpFormatFlags xmp_format_flags, guint32 padding, GError **error);
+
 /**
  * gexiv2_metadata_generate_xmp_packet:
  * @self: An instance of #GExiv2Metadata
@@ -702,22 +1086,39 @@ void			gexiv2_metadata_clear_xmp			(GExiv2Metadata *self);
  * Encode the XMP packet as a %NULL-terminated string.
  *
  * Returns: (transfer full) (allow-none): Encode the XMP packet and return as a %NULL-terminated string.
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_generate_xmp_packet() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_generate_xmp_packet)
 gchar*		gexiv2_metadata_generate_xmp_packet	(GExiv2Metadata *self, GExiv2XmpFormatFlags xmp_format_flags, guint32 padding);
 
+/**
+ * gexiv2_metadata_try_get_xmp_packet:
+ * @self: An instance of #GExiv2Metadata
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Returns: (transfer full) (allow-none): The currently-encoded XMP packet (see gexiv2_metadata_generate_xmp_packet).
+ *
+ * Since: 0.12.2
+ */
+gchar*			gexiv2_metadata_try_get_xmp_packet	(GExiv2Metadata *self, GError **error);
+
 /**
  * gexiv2_metadata_get_xmp_packet:
  * @self: An instance of #GExiv2Metadata
  *
  * Returns: (transfer full) (allow-none): The currently-encoded XMP packet (see gexiv2_metadata_generate_xmp_packet).
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_xmp_packet() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_xmp_packet)
 gchar*			gexiv2_metadata_get_xmp_packet		(GExiv2Metadata *self);
 
 /**
  * gexiv2_metadata_get_xmp_tags:
  * @self: An instance of #GExiv2Metadata
  *
- * Returns: (transfer full) (array zero-terminated=1): A list of the available XMP tags
+ * Returns: (transfer full) (array zero-terminated=1): A unique list of the available XMP tags
  */
 gchar**			gexiv2_metadata_get_xmp_tags		(GExiv2Metadata *self);
 
@@ -743,6 +1144,15 @@ gboolean        gexiv2_metadata_unregister_xmp_namespace (const gchar* name);
  */
 void            gexiv2_metadata_unregister_all_xmp_namespaces (void);
 
+/**
+ * gexiv2_metadata_get_xmp_namespace_for_tag:
+ * @tag: (in): Full tag name or XMP namespace identifier
+ * Returns: (transfer full): %NULL if there was no namespace registered for the tag, the URI of the namespace otherwise.
+ *
+ * Since: 0.12.2
+ */
+char* gexiv2_metadata_get_xmp_namespace_for_tag(const char* tag);
+
 /* IPTC functions */
 
 /**
@@ -767,7 +1177,7 @@ void			gexiv2_metadata_clear_iptc			(GExiv2Metadata *self);
  *
  * Query @self for a list of available IPTC tags
  *
- * Returns: (transfer full) (array zero-terminated=1): A list of the available IPTC tags
+ * Returns: (transfer full) (array zero-terminated=1): A unique list of the available IPTC tags
  */
 gchar**			gexiv2_metadata_get_iptc_tags		(GExiv2Metadata *self);
 
@@ -822,7 +1232,7 @@ gint gexiv2_metadata_get_metadata_pixel_height (GExiv2Metadata *self);
 /**
  * gexiv2_metadata_set_metadata_pixel_width:
  * @self: An instance of #GExiv2Metadata
- * @width: The width of the image as it should be put into the metadta.
+ * @width: The width of the image as it should be put into the metadata.
  *
  * Composite setter to update the image's metadata with @width
  */
@@ -831,7 +1241,7 @@ void gexiv2_metadata_set_metadata_pixel_width (GExiv2Metadata *self, gint width)
 /**
  * gexiv2_metadata_set_metadata_pixel_height:
  * @self: An instance of #GExiv2Metadata
- * @height: The width of the image as it should be put into the metadta.
+ * @height: The width of the image as it should be put into the metadata.
  *
  * Update the image's metadata with @height
  */
@@ -860,7 +1270,7 @@ void gexiv2_metadata_set_metadata_pixel_height (GExiv2Metadata *self, gint heigh
  * two-byte character codes for encoding.  However, it's still used here for legacy reasons.
  * </note>
  *
- * For fine-grained control, it's recommened to use Exiv2 tags directly rather than this method,
+ * For fine-grained control, it's recommended to use Exiv2 tags directly rather than this method,
  * which is more useful for quick or casual use.
  *
  * Returns: (transfer full) (allow-none): The photo's comment field.
@@ -882,7 +1292,7 @@ void			gexiv2_metadata_set_comment			(GExiv2Metadata *self, const gchar* comment
  * @self: An instance of #GExiv2Metadata
  *
  * This is a composite clear method that will clear a number of fields.  See
- * #gexiv2_metadata_get_comment for more informtion.
+ * #gexiv2_metadata_get_comment for more information.
  */
 void			gexiv2_metadata_clear_comment		(GExiv2Metadata *self);
 
@@ -904,6 +1314,8 @@ gboolean		gexiv2_metadata_get_exposure_time	(GExiv2Metadata *self, gint *nom, gi
  * @self: An instance of #GExiv2Metadata
  *
  * See <ulink url="https://en.wikipedia.org/wiki/F-number"></ulink> for more information.
+ * If Exif.Photo.FNumber does not exist, it will fall back to calculating the FNumber from
+ * Exif.Photo.ApertureValue (if available);
  *
  * Returns: The exposure Fnumber as a gdouble, or -1.0 if tag is not present or invalid.
  */
@@ -933,6 +1345,49 @@ gint			gexiv2_metadata_get_iso_speed		(GExiv2Metadata *self);
  * GPS functions
  */
 
+/**
+ * gexiv2_metadata_try_get_gps_longitude:
+ * @self: An instance of #GExiv2Metadata
+ * @longitude: (out): Variable to store the longitude value
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Query the longitude stored in the GPS tags of @self
+ *
+ * Returns: (skip): Boolean success value
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_get_gps_longitude			(GExiv2Metadata *self, gdouble *longitude, GError **error);
+
+/**
+ * gexiv2_metadata_try_get_gps_latitude:
+ * @self: An instance of #GExiv2Metadata
+ * @latitude: (out): Variable to store the latitude value
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Query the latitude stored in the GPS tags of @self
+ *
+ * Returns: (skip): Boolean success value
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_get_gps_latitude			(GExiv2Metadata *self, gdouble *latitude, GError **error);
+
+/**
+ * gexiv2_metadata_try_get_gps_altitude:
+ * @self: An instance of #GExiv2Metadata
+ * @altitude: (out): Variable to store the altitude value
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Convenience function to query the altitude stored in the GPS tags of the
+ * image
+ *
+ * Returns: (skip): Boolean success value
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_get_gps_altitude		(GExiv2Metadata *self, gdouble *altitude, GError **error);
+
 /**
  * gexiv2_metadata_get_gps_longitude:
  * @self: An instance of #GExiv2Metadata
@@ -941,7 +1396,10 @@ gint			gexiv2_metadata_get_iso_speed		(GExiv2Metadata *self);
  * Query the longitude stored in the GPS tags of @self
  *
  * Returns: (skip): Boolean success value
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_gps_longitude() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_gps_longitude)
 gboolean		gexiv2_metadata_get_gps_longitude			(GExiv2Metadata *self, gdouble *longitude);
 
 /**
@@ -952,7 +1410,10 @@ gboolean		gexiv2_metadata_get_gps_longitude			(GExiv2Metadata *self, gdouble *lo
  * Query the latitude stored in the GPS tags of @self
  *
  * Returns: (skip): Boolean success value
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_gps_latitude() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_gps_latitude)
 gboolean		gexiv2_metadata_get_gps_latitude			(GExiv2Metadata *self, gdouble *latitude);
 
 /**
@@ -960,13 +1421,32 @@ gboolean		gexiv2_metadata_get_gps_latitude			(GExiv2Metadata *self, gdouble *lat
  * @self: An instance of #GExiv2Metadata
  * @altitude: (out): Variable to store the altitude value
  *
- * Convenience functon to query the altitude stored in the GPS tags of the
+ * Convenience function to query the altitude stored in the GPS tags of the
  * image
  *
  * Returns: (skip): Boolean success value
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_gps_altitude() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_gps_altitude)
 gboolean		gexiv2_metadata_get_gps_altitude			(GExiv2Metadata *self, gdouble *altitude);
 
+/**
+ * gexiv2_metadata_try_get_gps_info:
+ * @self: An instance of #GExiv2Metadata
+ * @longitude: (out): Storage for longitude value
+ * @latitude: (out): Storage for latitude value
+ * @altitude: (out): Storage for altitude value
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Convenience function to query all available GPS information at once.
+ *
+ * Returns: (skip): Boolean success value.
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_get_gps_info			(GExiv2Metadata *self, gdouble *longitude, gdouble *latitude, gdouble *altitude, GError **error);
+
 /**
  * gexiv2_metadata_get_gps_info:
  * @self: An instance of #GExiv2Metadata
@@ -974,12 +1454,33 @@ gboolean		gexiv2_metadata_get_gps_altitude			(GExiv2Metadata *self, gdouble *alt
  * @latitude: (out): Storage for latitude value
  * @altitude: (out): Storage for altitude value
  *
- * Convenience functon to query all available GPS information at once.
+ * Convenience function to query all available GPS information at once.
  *
  * Returns: (skip): Boolean success value.
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_get_gps_info() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_get_gps_info)
 gboolean		gexiv2_metadata_get_gps_info				(GExiv2Metadata *self, gdouble *longitude, gdouble *latitude, gdouble *altitude);
 
+/**
+ * gexiv2_metadata_try_set_gps_info:
+ * @self: An instance of #GExiv2Metadata
+ * @longitude: Longitude value to set or replace current value
+ * @latitude: Latitude value to set or replace current value
+ * @altitude: Altitude value to set or replace current value
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Convenience function to create a new set of simple GPS data. Warning: Will remove any other
+ * GPS information that is currently set. See gexiv2_metadata_update_gps_info() for
+ * just modifying the GPS data.
+ *
+ * Returns: (skip): Boolean success value.
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_set_gps_info			(GExiv2Metadata *self, gdouble longitude, gdouble latitude, gdouble altitude, GError **error);
+
 /**
  * gexiv2_metadata_set_gps_info:
  * @self: An instance of #GExiv2Metadata
@@ -987,18 +1488,71 @@ gboolean		gexiv2_metadata_get_gps_info				(GExiv2Metadata *self, gdouble *longit
  * @latitude: Latitude value to set or replace current value
  * @altitude: Altitude value to set or replace current value
  *
- * Convenience functon to query all available GPS information at once.
+ * Convenience function to create a new set of simple GPS data. Warning: Will remove any other
+ * GPS information that is currently set. See gexiv2_metadata_update_gps_info() for
+ * just modifying the GPS data.
  *
  * Returns: (skip): Boolean success value.
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_set_gps_info() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_set_gps_info)
 gboolean		gexiv2_metadata_set_gps_info				(GExiv2Metadata *self, gdouble longitude, gdouble latitude, gdouble altitude);
 
+/**
+ * gexiv2_metadata_try_update_gps_info:
+ * @self: An instance of #GExiv2Metadata
+ * @longitude: Longitude value to set or replace current value
+ * @latitude: Latitude value to set or replace current value
+ * @altitude: Altitude value to set or replace current value
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Convenience function to update longitude, latitude and altitude at once.
+ *
+ * Returns: (skip): Boolean success value.
+ *
+ * Since: 0.12.2
+ */
+gboolean		gexiv2_metadata_try_update_gps_info			(GExiv2Metadata *self, gdouble longitude, gdouble latitude, gdouble altitude, GError **error);
+
+/**
+ * gexiv2_metadata_update_gps_info:
+ * @self: An instance of #GExiv2Metadata
+ * @longitude: Longitude value to set or replace current value
+ * @latitude: Latitude value to set or replace current value
+ * @altitude: Altitude value to set or replace current value
+ *
+ * Convenience function to update longitude, latitude and altitude at once.
+ *
+ * Returns: (skip): Boolean success value.
+ *
+ * Since: 0.12.1
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_update_gps_info() instead.
+ */
+G_DEPRECATED_FOR(gexiv2_metadata_try_update_gps_info)
+gboolean		gexiv2_metadata_update_gps_info				(GExiv2Metadata *self, gdouble longitude, gdouble latitude, gdouble altitude);
+
+/**
+ * gexiv2_metadata_try_delete_gps_info:
+ * @self: An instance of #GExiv2Metadata
+ * @error: (allow-none): A return location for a #GError or %NULL
+ *
+ * Removes all GPS metadata from the loaded image
+ *
+ * Since: 0.12.2
+ */
+void			gexiv2_metadata_try_delete_gps_info			(GExiv2Metadata *self, GError **error);
+
 /**
  * gexiv2_metadata_delete_gps_info:
  * @self: An instance of #GExiv2Metadata
  *
  * Removes all GPS metadata from the loaded image
+ *
+ * Deprecated: 0.12.2: Use gexiv2_metadata_try_delete_gps_info() instead.
  */
+G_DEPRECATED_FOR(gexiv2_metadata_try_delete_gps_info)
 void			gexiv2_metadata_delete_gps_info			(GExiv2Metadata *self);
 
 /*
